"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongoBaseRepository = void 0;
const mongoose_1 = __importDefault(require("mongoose"));
const error_1 = require("../components/error");
class MongoBaseRepository {
    constructor(collection) {
        this.collection = collection;
        this.options = { timestamps: true };
        this.getInternalServerError = (errorPrefix, error, item) => {
            console.log("*********************** INTERNALLLLLLLL ******************8");
            return new error_1.InternalServerError(errorPrefix + "_" + this._model.modelName, error.message, { item, error });
        };
        this.schema = new mongoose_1.default.Schema(this.definition(), this.options);
        this.initiateIndexes();
        this.initiatePlugins();
        this._model = mongoose_1.default.model(this.constructor.name, this.schema, this.collection);
    }
    initiateIndexes() {
    }
    initiatePlugins() {
    }
    create(item) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this._model.create(item);
            }
            catch (e) {
                throw this.getInternalServerError("insert", e, item);
            }
        });
    }
    createOrFind(item) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.create(item);
            }
            catch (e) {
                if (e.details && e.details.error && e.details.error.code === 11000) {
                    return (yield this.findOne(item));
                }
                else {
                    throw this.getInternalServerError("create_or_find", e, "CREATE_OR_FIND");
                }
            }
        });
    }
    findOrCreate(item) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let entry = yield this.findOne(item);
                if (!entry) {
                    entry = yield this.create(item);
                }
                return entry;
            }
            catch (e) {
                throw this.getInternalServerError("create_or_find", e, "CREATE_OR_FIND");
            }
        });
    }
    insertMany(items) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this._model.insertMany(items);
            }
            catch (e) {
                throw this.getInternalServerError("insert_many", e, items);
            }
        });
    }
    update(id, item) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this._model.findByIdAndUpdate(id, item);
            }
            catch (e) {
                throw this.getInternalServerError("update", e, item);
            }
        });
    }
    updateMany(condition, item) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this._model.updateMany(condition, item);
            }
            catch (e) {
                throw this.getInternalServerError("update", e, item);
            }
        });
    }
    delete(id) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return !!(yield this._model.findByIdAndRemove(id));
            }
            catch (e) {
                throw this.getInternalServerError("delete", e, "DELETE");
            }
        });
    }
    deleteOne(item) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this._model.findOneAndRemove(item);
            }
            catch (e) {
                throw this.getInternalServerError("delete_one", e, "DELETE");
            }
        });
    }
    deleteMany(item) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this._model.deleteMany(item);
            }
            catch (e) {
                throw this.getInternalServerError("delete_many", e, "DELETE_MANY");
            }
        });
    }
    deleteAll() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this._model.remove({});
            }
            catch (e) {
                throw this.getInternalServerError("delete_all", e, "DELETE_ALL");
            }
        });
    }
    find(item) {
        return this._model.find(item);
    }
    findOne(item) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this._model.findOne(item);
            }
            catch (e) {
                throw this.getInternalServerError("find_one", e, item);
            }
        });
    }
    findById(id, select) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield (select ? this._model.findById(id).select(select) : this._model.findById(id));
            }
            catch (e) {
                throw this.getInternalServerError("find_by_id", e, "FIND");
            }
        });
    }
    findAndUpdate(condition, item, options = { new: true }) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this._model.findOneAndUpdate(condition, item, options);
            }
            catch (e) {
                throw this.getInternalServerError("find_and_updated", e, item);
            }
        });
    }
    paginate(filter, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // @ts-ignore
            return yield this._model.paginate(filter, options);
        });
    }
    aggregatePaginate(filter, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // @ts-ignore
            return yield this._model.aggregatePaginate(this._model.aggregate(filter), options);
        });
    }
    populate(filter, populate) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this._model.populate(filter, populate);
            }
            catch (e) {
                throw this.getInternalServerError("aggregate", e, "Populate Query");
            }
        });
    }
    findOnePopulate(item, populate, select) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return (yield this._model.findOne(item).populate(populate).select(select));
            }
            catch (e) {
                throw this.getInternalServerError("find_one_populate", e, item);
            }
        });
    }
    findAndPopulate(item, populate, select) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return (yield this._model.find(item).populate(populate).select(select));
            }
            catch (e) {
                throw this.getInternalServerError("find_one_populate", e, item);
            }
        });
    }
    getCount(condition) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return +(yield this._model.count(condition));
            }
            catch (e) {
                throw this.getInternalServerError("count", e, "Count Query");
            }
        });
    }
    aggregate(condition, allowDiskUse = false) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const promise = this._model.aggregate(condition);
                return allowDiskUse ? yield promise.allowDiskUse(true) : yield promise;
            }
            catch (e) {
                throw this.getInternalServerError("aggregate", e, "Aggregate Query");
            }
        });
    }
}
exports.MongoBaseRepository = MongoBaseRepository;
//# sourceMappingURL=mongo.js.map